@@ -63,28 +63,23 @@ create view jaaql__user_latest_password as (
 );
 
 create table jaaql__node (
-    id uuid primary key not null default gen_random_uuid(),
-    name varchar(255) not null,
+    name varchar(255) primary key not null,
     description varchar(255) not null,
     port integer not null,
 	address varchar(255) not null,
 	interface_class varchar(20) not null,
 	deleted timestamptz
 );
-CREATE UNIQUE INDEX jaaql__node_unq_name
-    ON jaaql__node (name) WHERE (deleted is null);
+
 CREATE UNIQUE INDEX jaaql__node_unq
     ON jaaql__node (address, port) WHERE (deleted is null);
 
 create table jaaql__database (
-    id uuid primary key not null default gen_random_uuid(),
-    node uuid not null references jaaql__node,
+    node varchar(255) not null references jaaql__node on update cascade on delete cascade,
 	name varchar(255) not null,
-	deleted timestamptz
+	deleted timestamptz,
+	PRIMARY KEY (node, name)
 );
---Cannot duplicate name on node
-CREATE UNIQUE INDEX jaaql__database_unq
-    ON jaaql__database (name, node) WHERE (deleted is null);
 
 create table jaaql__application (
     name varchar(64) not null primary key,
@@ -94,17 +89,17 @@ create table jaaql__application (
 );
 
 INSERT INTO jaaql__application (name, description, url) VALUES ('console', 'The console application', '');
+INSERT INTO jaaql__application (name, description, url) VALUES ('manager', 'The administration panel for JAAQL', '');
 
 create table jaaql__application_parameter (
     application varchar(64) not null,
     name varchar(64) not null,
     description varchar(255) not null,
-    is_node boolean not null default false,
-    PRIMARY KEY (application, name, is_node),  -- Fake primary key used for checking node match
+    PRIMARY KEY (application, name),
     FOREIGN KEY (application) references jaaql__application on delete cascade on update cascade
 );
--- Real primary key here. Alternative solution is to use a trigger which is messy
-CREATE UNIQUE INDEX jaaql__application_parameter_unq ON jaaql__application_parameter (application, name);
+INSERT INTO jaaql__application_parameter (application, name, description) VALUES ('console', 'node', 'The node which the console will run against');
+INSERT INTO jaaql__application_parameter (application, name, description) VALUES ('manager', 'node', 'A jaaql node which the app can manage');
 
 create table jaaql__application_configuration (
     application varchar(64) not null,
@@ -113,36 +108,30 @@ create table jaaql__application_configuration (
     PRIMARY KEY (application, name),
     FOREIGN KEY (application) references jaaql__application on delete cascade on update cascade
 );
+INSERT INTO jaaql__application_configuration (application, name, description) VALUES ('manager', 'host', 'The host jaaql node');
 
 create table jaaql__application_argument (
     application varchar(64) not null,
     configuration varchar(64) not null,
-    database uuid,
-    node uuid,
-
-    -- generated column to avoid trigger. Can now use a foreign key
-    is_node boolean GENERATED ALWAYS AS (node is not null) STORED,
-
-    CHECK ((database is null) <> (node is null)),
+    database varchar(255) not null,
+    node varchar(255) not null,
     parameter varchar(64) not null,
     PRIMARY KEY (application, parameter, configuration),
-    FOREIGN KEY (application, configuration) references jaaql__application_configuration
+    FOREIGN KEY (application, configuration) references jaaql__application_configuration(application, name)
         on delete cascade on update cascade,
     FOREIGN KEY (application, parameter) references jaaql__application_parameter(application, name) ON DELETE CASCADE ON UPDATE CASCADE,
-    FOREIGN KEY (application, parameter, is_node) references jaaql__application_parameter(application, name, is_node),
-    FOREIGN KEY (database) references jaaql__database,
-    FOREIGN KEY (node) references jaaql__node
+    FOREIGN KEY (database, node) references jaaql__database(name, node) on delete cascade on update cascade,
+    FOREIGN KEY (node) references jaaql__node (name) on delete cascade on update cascade
 );
 
---Configurations with any missing parameters highlighted
 create view jaaql__configuration_argument as (
     SELECT
         db_conf.application as application,
     	db_conf.name as configuration,
     	db_param.name as parameter_name,
         db_param.description as parameter_description,
-    	coalesce(db_arg.database, db_arg.node) as argument,
-        db_arg.is_node
+    	db_arg.database as database,
+        db_arg.node as node
     FROM
     	jaaql__application_configuration db_conf
     INNER JOIN jaaql__application_parameter db_param ON db_conf.application = db_param.application
@@ -150,8 +139,7 @@ create view jaaql__configuration_argument as (
         ON db_conf.application = db_arg.application AND
            db_conf.name = db_arg.configuration AND
            db_arg.parameter = db_param.name
-    LEFT JOIN jaaql__database jd on db_arg.database = jd.id AND jd.deleted is null
-    LEFT JOIN jaaql__node jn on db_arg.node = jn.id AND jn.deleted is null
+    LEFT JOIN jaaql__database jd on db_arg.database = jd.name
 );
 
 -- Complete configurations. Where each parameter for the application has an associated argument
@@ -162,7 +150,7 @@ create view jaaql__complete_configuration as (
          jaaql__configuration_argument
     WHERE (application, configuration, true) IN (
         SELECT
-            application, configuration, bool_and(argument is not null)
+            application, configuration, bool_and(database is not null)
         FROM
              jaaql__configuration_argument
         GROUP BY
@@ -170,39 +158,26 @@ create view jaaql__complete_configuration as (
     )
 );
 
-create table jaaql__authorization_application (
+create table jaaql__authorization_configuration (
     application varchar(64) not null references jaaql__application on delete cascade on update cascade,
+    configuration varchar(64) not null,
     role        varchar(254) not null default '',
-    primary key (application, role)
+    primary key (application, configuration, role),
+    foreign key (application, configuration) references jaaql__application_configuration on delete cascade on update cascade
 );
 
-INSERT INTO jaaql__authorization_application (application) VALUES ('console');
-
-create table jaaql__authorization_node (
+create table jaaql__credentials_node (
     id uuid PRIMARY KEY not null default gen_random_uuid(),
-    node uuid not null references jaaql__node,
+    node varchar(255) not null references jaaql__node,
     role varchar(254),
     deleted timestamptz,
     db_encrypted_username varchar(254) not null,
 	db_encrypted_password varchar(254) not null,
 	precedence int not null default 0
 );
-CREATE UNIQUE INDEX jaaql__authorization_node_unq
-    ON jaaql__authorization_node (node, role) WHERE (deleted is null);
+CREATE UNIQUE INDEX jaaql__credentials_node_unq
+    ON jaaql__credentials_node (node, role) WHERE (deleted is null);
 
-create table jaaql__authorization_node_database (
-    "authorization" uuid not null references jaaql__authorization_node,
-    "database" uuid not null references jaaql__database,
-    PRIMARY KEY ("authorization", "database")
-);
-
-create table jaaql__authorization_node_database_upsert (
-    "authorization" uuid not null references jaaql__authorization_node,
-    "database" uuid not null references jaaql__database,
-    PRIMARY KEY ("authorization", "database")
-);
-
---TODO add node authorization and nullable db associated
 create table jaaql__log (
     id uuid primary key not null default gen_random_uuid(),
     the_user uuid,
@@ -221,10 +196,10 @@ create table jaaql__log (
 
 create table jaaql__log_db_auth (
     log uuid not null,
-    auth uuid not null,
-    PRIMARY KEY (log, auth),
+    credentials uuid not null,
+    PRIMARY KEY (log, credentials),
     FOREIGN KEY (log) REFERENCES jaaql__log,
-    FOREIGN KEY (auth) REFERENCES jaaql__authorization_node
+    FOREIGN KEY (credentials) REFERENCES jaaql__credentials_node
 );
 
 create view jaaql__my_logs as (
@@ -259,36 +234,28 @@ grant select on jaaql__my_ips to public;
 
 create view jaaql__my_configurations as (
     SELECT
-        sub.application,
-        sub.configuration,
+        DISTINCT
+        ac.application,
+        ac.name as configuration,
         ap.description as application_description,
         ac.description as configuration_description
     FROM
-        (SELECT
-            app_conf.application as application,
-            app_conf.name as configuration
-        FROM
-             jaaql__authorization_application app_auth
-        INNER JOIN jaaql__application_configuration app_conf ON app_auth.application = app_conf.application
-        WHERE
-            (app_conf.application, app_conf.name, true) IN (
-                SELECT conf_arg.application,
-                       conf_arg.configuration,
-                       bool_and(node_auth.id is not null)
-                FROM jaaql__complete_configuration conf_arg
-                LEFT JOIN jaaql__authorization_node_database acd ON
-                    conf_arg.argument = acd.database
-                LEFT JOIN jaaql__authorization_node node_auth ON
-                    ((acd."authorization" is not null AND acd."authorization" = node_auth.id) or
-                    (conf_arg.argument = node_auth.node)) AND node_auth.deleted is null AND (node_auth.role = '' or pg_has_role(node_auth.role, 'MEMBER'))
-                INNER JOIN jaaql__authorization_application app_auth ON app_auth.application = conf_arg.application AND (app_auth.role = '' or pg_has_role(app_auth.role, 'MEMBER'))
-                GROUP BY conf_arg.application, conf_arg.configuration
-            )
-        GROUP BY app_conf.application, app_conf.name) as sub
-    INNER JOIN jaaql__application_configuration ac ON ac.name = sub.configuration
-    INNER JOIN jaaql__application ap on ap.name = sub.application);
+         jaaql__application_configuration ac
+    INNER JOIN jaaql__authorization_configuration auth_conf ON ac.name = auth_conf.configuration AND ac.application = auth_conf.application AND (auth_conf.role = '' or pg_has_role(auth_conf.role, 'MEMBER'))
+    INNER JOIN jaaql__application ap on ap.name = ac.application);
 grant select on jaaql__my_configurations to public;
 
+create view jaaql__my_applications as (
+    SELECT
+        DISTINCT
+        ap.name,
+        ap.description,
+        ap.url
+    FROM jaaql__application ap
+    INNER JOIN jaaql__my_configurations mc ON mc.application = ap.name
+);
+grant select on jaaql__my_applications to public;
+
 -- Not my. Called through jaaql connection. Need to keep username and password secret
 create view jaaql__their_authorized_configurations as (
     SELECT
@@ -296,22 +263,38 @@ create view jaaql__their_authorized_configurations as (
         comc.configuration,
         comc.parameter_name,
         comc.parameter_description,
-        jand.db_encrypted_username as username,
-        jand.db_encrypted_password as password,
-        jd.id as database,
-        jd.name as name,
-        jand.node as node,
+        jcred.db_encrypted_username as username,
+        jcred.db_encrypted_password as password,
+        comc.database,
+        comc.node as node,
+        jcred.id as cred_id,
         jn.port,
         jn.address,
-        jand.precedence,
-        jand.role as node_role,
-        ja.role as application_role
+        jcred.precedence,
+        jcred.role as node_role,
+        auth_conf.role as conf_role
     FROM jaaql__complete_configuration comc
-        LEFT JOIN jaaql__authorization_node_database jandat ON comc.argument = jandat.database
-        LEFT JOIN jaaql__database jd ON jandat.database = jd.id
-        INNER JOIN jaaql__authorization_node jand ON (jand.node = comc.argument or jand.id = jandat."authorization") AND jand.deleted is null
-        INNER JOIN jaaql__authorization_application ja on comc.application = ja.application
-        INNER JOIN jaaql__node jn on jand.node = jn.id
+        INNER JOIN jaaql__credentials_node jcred ON (jcred.node = comc.node) AND jcred.deleted is null
+        INNER JOIN jaaql__authorization_configuration auth_conf on comc.application = auth_conf.application AND comc.configuration = auth_conf.configuration
+        INNER JOIN jaaql__node jn on comc.node = jn.name
+);
+
+create view jaaql__their_single_authorized_wildcard_node as (
+    SELECT
+        jn.name as node,
+        jn.address,
+        jn.port,
+        cred.role,
+        cred.precedence,
+        cred.db_encrypted_username as username,
+        cred.db_encrypted_password as password
+    FROM
+        jaaql__credentials_node cred
+    INNER JOIN
+        jaaql__database jd on cred.node = jd.node
+    INNER JOIN
+        jaaql__node jn on cred.node = jn.name
+    WHERE jd.name = '*' AND jd.deleted is null AND cred.deleted is null AND jn.deleted is null
 );
 
 -- DO NOT CHANGE FUNCTION BELOW WITHOUT CONSIDERING SECURITY!!!
@@ -339,7 +322,7 @@ $$
 BEGIN
     EXECUTE 'DROP ROLE ' || quote_ident(delete_email);
     UPDATE jaaql__user SET deleted = current_timestamp WHERE email = delete_email;
-    UPDATE jaaql__authorization_node SET deleted = current_timestamp WHERE role = delete_email;
+    UPDATE jaaql__credentials_node SET deleted = current_timestamp WHERE role = delete_email;
 END
 $$ language plpgsql;
 
@@ -358,20 +341,27 @@ $$
 DECLARE
     actual_alias text;
 BEGIN
-    SELECT coalesce(alias, email) into actual_alias FROM jaaql__user WHERE id = userid;
+    SELECT coalesce(alias, email) into actual_alias FROM jaaql__user WHERE id = userid::uuid;
     return actual_alias;
 END
 $$ language plpgsql;
 
-create function jaaql__create_node(node_name text, node_address text, node_port integer, node_description text) returns uuid as
+create function jaaql__create_node(node_name text, node_address text, node_port integer, node_description text) returns character varying as
 $$
 DECLARE
-    node_id uuid;
+    node_id character varying;
 BEGIN
-    INSERT INTO jaaql__node (name, description, port, address, interface_class) VALUES (node_name, coalesce(node_description, 'The ' || node_name || ' node'), node_port, node_address, 'DBPGInterface') RETURNING id into node_id;
-    INSERT INTO jaaql__application_configuration (application, name, description) VALUES ('console', 'Console ' || node_name || ' node', 'Console access to the ' || node_name || ' node');
-    INSERT INTO jaaql__application_parameter (application, name, description, is_node) VALUES ('console', 'node', 'The node which the console will run against', true);
-    INSERT INTO jaaql__application_argument (application, configuration, node, parameter) SELECT 'console', 'Console ' || node_name || ' node', id, 'node' FROM jaaql__node;
+    INSERT INTO jaaql__node (name, description, port, address, interface_class) VALUES (coalesce(node_name, node_address || node_port), coalesce(node_description, 'The ' || node_name || ' node'), node_port, node_address, 'DBPGInterface') RETURNING name into node_id;
+    INSERT INTO jaaql__database (node, name) VALUES (node_id, '*');
+    INSERT INTO jaaql__application_configuration (application, name, description) VALUES ('console', node_id, 'Console access to the ' || node_id || ' node');
+    INSERT INTO jaaql__application_argument (application, configuration, node, database, parameter) VALUES ('console', node_id, node_id, '*', 'node');
     return node_id;
 END
 $$ language plpgsql;
+
+create or replace function jaaql__delete_node(node_name text) returns void as
+$$
+BEGIN
+    UPDATE jaaql__node SET "name" = (left("name", 180) || '_deleted_') || current_timestamp::text, deleted = current_timestamp WHERE name = node_name;
+END
+$$ language plpgsql;
